#include <iostream>
#include "ColouredPlane.h"

ColouredPlane::ColouredPlane(const PointSet &set) : SparseGraph(Q, D), set(set) {

    // first colour: points in the set
    // second colour: points not in the set
    int pos1 = 0;
    int pos2 = V;
    for (int pt = 0; pt < V; pt++) {
        if (set.contains(pt)) {
            lab[pos1] = pt;
            pos1++;
        } else {
            pos2--;
            lab[pos2] = pt;
        }
    }
    // third colour: lines
    for (int ln = V; ln < 2 * V; ln++) {
        lab[ln] = ln;
    }

    // TODO: use weights to colour the lines? (Nauty will do that for us anyhow)

    for (int &i : ptn) {
        i = 1;
    }
    ptn[set.getSize() - 1] = 0;
    ptn[V - 1] = 0;
    ptn[2 * V - 1] = 0;

}

SparseGraph *ColouredPlane::canonicalForm() {
    DEFAULTOPTIONS_SPARSEGRAPH(options);
    options.defaultptn = false; // use colours
    statsblk stats{0,};
    SparseGraph sg{Q, D};
    SparseGraph *canonsg = new SparseGraph{2 * V, 2 * V * (Q + 1)};
    options.getcanon = true;
    sparsenauty(&sg, lab, ptn, orbits, &options, &stats, canonsg);
    return canonsg;
}

SparseGraph *ColouredPlane::isNautyCanonical(int pt, const Weights &weights) {
    SparseGraph *canonsg = canonicalForm();

    // Check whether this point is mapped to the first orbit of the same invariant value
    // as pt. We know that this is the highest possible value, but this is not used.

    // because of the way thing are coloured, the first size positions in lab
    // correspond to points of the set!
    int value = weights.invariantOf(pt);
    int i=0;
    while (i < set.getSize() && weights.invariantOf(lab[i]) != value) {
        i ++;
    }
    if (orbits[pt] == orbits[lab[i]]) {
        return canonsg;
    } else {
        delete canonsg;
        return nullptr;
    }
}

int ColouredPlane::stabilizerSize() {
    DEFAULTOPTIONS_SPARSEGRAPH(options);
    options.defaultptn = false; // use colours
    statsblk stats{0,};
    SparseGraph sg{Q, D};
    sparsenauty(&sg, lab, ptn, orbits, &options, &stats, nullptr);
    return (int)stats.grpsize1;
}

void ColouredPlane::printOrbits() {
    int nr = 0;
    int total = 0;
    // point orbits
    while (total < V) {
        // find next orbit
        while (orbits[nr] < nr) {
            nr ++;
        }
        std::cout << "[ ";
        for (int pt = 0; pt < V; pt++) {
            if (orbits[pt] == nr) {
                std::cout << pt << " ";
                total ++;
            }
        }
        std::cout << "] ";
        nr ++;
    }
    std::cout << " :: ";
    // line orbits
    while (total < 2*V) {
        // find next orbit
        while (orbits[nr] < nr) {
            nr ++;
        }
        std::cout << "[ ";
        for (int ln = V; ln < 2 * V; ln++) {
            if (orbits[ln] == nr) {
                std::cout << (ln - V) << " ";
                total ++;
            }
        }
        std::cout << "] ";
        nr ++;
    }

    // line orbits - TODO
}
