#ifndef BSETSALT_COLOUREDPLANE_H
#define BSETSALT_COLOUREDPLANE_H

#include "SparseGraph.h"
#include "PointSet.h"
#include "Weights.h"

// Represents the current projective plane, coloured by a set of points
// Provides methods that interface to Nauty to provide canonicity tests and group computations

class ColouredPlane : SparseGraph {

private:
    const PointSet &set;

    int lab[2 * V];
    int ptn[2 * V];
    int orbits[2 * V];

public:
    ColouredPlane (const PointSet &set);

    // return the corresponding canonical form
    SparseGraph* canonicalForm();

    // check whether the given point is canonical in the current set, according to Nauty
    // Returns the null pointer if not canonical. Returns the canonical form (for further use)
    // when the point is canonical.
    //
    // IMPORTANT
    // - Only call when the invariants for that points are computes, and stored in weights.
    // - pt must be known to have the highest invariant value of all points in the set
    SparseGraph* isNautyCanonical (int pt, const Weights &weights);

    // size of the stabilizer group of the associated point set
    int stabilizerSize();

    // print the orbits to standard out - on one line with no linefeed
    void printOrbits(); // TODO not only to std::cout
};


#endif //BSETSALT_COLOUREDPLANE_H
