#ifndef BSETSALT_GENERATOR_H
#define BSETSALT_GENERATOR_H

#include "options.h"
#include "PointSet.h"
#include "Weights.h"

class Generator {

public:
    void generate () ;

    PointSet currentSet;

private:

    // Keeps track of all weights for the current set
    Weights weights {currentSet};

    // recursive version of generate
    void gen ();

    // check whether the currently generated set is complete
    bool isComplete();

    // is this a valid subset of a possible result? (pt = last point added)
    bool isValid();

    // is this a point that can be added to the current set so that it still remains valid?
    bool isValid (int pt);

    // is the given point canonical in the set (wrt. isomorph freeness)
    CheckResult checkMaybeCanonical(int pt);

    // add a point (which is not already in the current set)
    void add (int pt);

    // remove a point (which is already in the current set)
    void remove (int pt);

public:

    // print statistics to std::cerr
    void printStatistics();

    void processArgs(int argc, char *argv[]);

private:
    // nr of results generated
    long totalCount = 0;

    std::array<long, V> counts = {0,}; // too large, but who cares?

    // Parameters to use when splitting process for parallelization
    int splitDepth = V + 1; // default = do not split
    int splitIndex = 0;
    int splitModulus = 0;
    int splitCounter = 0;

};


#endif //BSETSALT_GENERATOR_H
