#include "SparseGraph.h"
#include <nauty.h>

SparseGraph::SparseGraph(int nrOfVertices, int nrOfDirectedEdges) : sparsegraph () {
    vlen = dlen = nv = nrOfVertices;
    elen = nde = nrOfDirectedEdges;
    wlen = 0;

    d = new int[nrOfVertices];
    v = new size_t[nrOfVertices];
    e = new int[nrOfDirectedEdges];
    w = nullptr;
}

SparseGraph::~SparseGraph() {
    delete d;
    delete v;
    delete e;
}

SparseGraph::SparseGraph(int q, const int dset[]) : SparseGraph(2*(q * q + q + 1), 2*(q * q + q + 1)*(q+1)) {

    // note: copied from a more general version in which point and lines do not have constant degree

    int nr = q * q + q + 1;
    int vpos = 0;
    size_t epos = 0;

    // points are stored first, and then lines, with offset nr

    // for each point, store incidence with each line
    for (int pt = 0; pt < nr; pt++) {
        v[vpos] = epos;
        for (int i = 0; i <= q; i++) {
            e[epos] = (pt + nr - dset[i]) % nr + nr;
            epos++;
        }
        d[vpos] = (int) (epos - v[vpos]);
        vpos++;
    }

    // for each line store incidence with each point
    for (int ln = 0; ln < nr; ln++) {
        v[vpos] = epos;
        for (int i = 0; i <= q; i++) {
            e[epos] = (ln + dset[i]) % nr;
            epos++;
        }
        d[vpos] = (int) (epos - v[vpos]);
        vpos++;
    }

    nde = epos;
}

bool SparseGraph::operator==(const SparseGraph &other) const {
    return aresame_sg((sparsegraph *) this, (sparsegraph *)&other);
}

bool SparseGraph::operator!=(const SparseGraph &other) const {
    return !aresame_sg((sparsegraph *) this, (sparsegraph *)&other);
}
