#include "Weights.h"
#include "PointSet.h"

void Weights::add(int pt) {
    for (int i = 0; i <= Q; i++) {
        int ln = (pt + V - D[i]) % V;
        weights[ln]++;
    }
    // TODO introduce abstraction for 'all lines through a point' and 'all points through a line'
}

void Weights::remove(int pt) {
    for (int i = 0; i <= Q; i++) {
        int ln = (pt + V - D[i]) % V;
        weights[ln]--;
    }
}

bool Weights::isBlocking() const {
    for (const auto &weight : weights) {
        if (weight == 0) {
            return false;
        }
    }
    return true;
}

bool Weights::isSuperfluous(int pt) const {
    for (int i = 0; i <= Q; i++) {
        int ln = (pt + V - D[i]) % V;
        if (weights[ln] == 0) {
            return false;
        }
    }
    return true;
}

bool Weights::isFat(int pt) const {
    // TODO: very similar to isSuperfluous
    for (int i = 0; i <= Q; i++) {
        int ln = (pt + V - D[i]) % V;
        if (weights[ln] <= 1) {
            return false;
        }
    }
    return true;
}

bool Weights::isMinimal() const {
    for (int pt = 0; pt < V; pt++) {
        if (set.contains(pt) && isFat(pt)) {
            // TODO: abstract loop over all points of a set?
            return false;
        }
    }
    return true;
}

int Weights::computeInvariant(int pt) const {
    // TODO use hash instead of square
    int total = 0;
    for (int i = 0; i <= Q; i++) {
        int ln = (pt + V - D[i]) % V;
        total += hashes[weights[ln]];
    }
    return total;
}

void Weights::computeInvariants() {
    for (int pt = 0; pt < V; pt++) {
        if (set.contains(pt)) {
            invariants[pt] = computeInvariant(pt);
        }
    }
}

CheckResult Weights::checkCanonical(int pt) {
    // if the point has the highest unique invariant, then it is canonical
    // if there are no unique invariants, the point has to have the highest
    // invariant value to be able to be canonical.

    // Step 1. Check for higest unique invaruant
    //////

    std::array<bool, V> removed{false,};
    // this array keeps track of which points should still be visited,
    // after the loop, only for the unique invariants the value remains false

    int nrOfUnique = 0;
    for (int i = 0; i < V; i++) {
        if (set.contains(i) && !removed[i]) {
            bool found = false;
            for (int j = i + 1; j < V; j++) {
                if (set.contains(j) && !removed[j] && invariants[i] == invariants[j]) {
                    found = true;
                    removed[j] = true;
                }
            }
            if (found) {
                removed[i] = true;
            } else {
                // i has a unique invariant value
                nrOfUnique++;
                if (invariants[i] > invariants[pt]) {
                    return NOT_CANONICAL; // there is at least one unique invariant value, but it is larger than for the requested point
                }
            }
        }
    }
    if (nrOfUnique != 0) {
        if (removed[pt]) {
            return NOT_CANONICAL;
        } else {
            return CANONICAL;
        }
    }

    // Step 2. Highest invariant value
    /////
    for (int i = 0; i < V; i++) {
        if (set.contains(i) && invariants[i] > invariants[pt]) {
            return NOT_CANONICAL;
        }
    }

    return UNKNOWN; // must defer to Nauty...
}

int Weights::computeSetInvariant() {
    int total = 0;
    for (int pt = 0; pt < V; pt++) {
        if (set.contains(pt)) {
            total += invariants[pt] * invariants[pt];
        }
    }
    return total;
}

Weights::Weights(const PointSet& set) : set(set) {
    int val = 1;
    for (int i=0; i <= Q+1; i++) {
        hashes[i] = val;
        val *= 10;
    }
}

