#ifndef BSETSALT_WEIGHTS_H
#define BSETSALT_WEIGHTS_H

#include <array>
#include "options.h"
#include "PointSet.h"

enum CheckResult{ CANONICAL, NOT_CANONICAL, UNKNOWN };

/**
 * Keeps track of the weights of the lines in the plane wrt. the current point set
 */
class Weights {

private:
    std::array<int,V> weights {0, };
    std::array<int,V> invariants {0, };

    const PointSet &set;

public:

    Weights (const PointSet &set);

    void add (int pt);
    void remove (int pt);

    int invariantOf (int pt) const { return invariants[pt]; }

    // A set is a blocking set if all lines have weight ≥ 1
    bool isBlocking() const;

    // A set is 'minimal' if there is no point for which all lines have weight > 1
    bool isMinimal() const ;

    // A point is superfluous iff all lines through it have weight ≥ 1
    bool isSuperfluous (int pt) const;

    // Compute and store the invariant values of all points of the set
    void computeInvariants () ;

    // Determine canonicity using the computed invariants
    // Call computeInvariants before calling this method
    CheckResult checkCanonical (int pt);

    // Compute the invariant for the complete set. A prior call to computeInvariants is necessary
    int computeSetInvariant () ;

private:

    // A point is fat iff all lines through it have weight > 1
    bool isFat(int pt) const;

    // Compute invariant for the given point
    int computeInvariant (int pt) const;

    //
    std::array<int,Q+2> hashes {0, };
};


#endif //BSETSALT_WEIGHTS_H
