Generator for minimal blocking sets in small Desarguesian projective planes
===

This is the source code of the generation program used in the paper

* "Classification of minimal blocking sets in PG(2, 9)", 
  *Designs, Codes and Cryptography*, (submitted 2022).

This program provides a (much faster) alternative to the generator used in the earlier paper
"Classification of minimal blocking sets in small Desarguesian projective planes",
  *Journal of Combinatorial Designs*, Vol 30(8), 2022.

The program is written in C++ and was developed using the CLion IDE which uses
`cmake` as build tool and `clang` as compiler. (It should be possible to rebuild
the software outside of CLion, as long as you have `cmake` and `clang` available.)

A few unit tests are provided with the source code - see below - but they are not
needed to be able to run the main program.

#### Main program

The main program - called `main` - generates all minimal blocking sets in the
Desarguesian plane of a given order. Currently orders 3, 4, 5, 6, 7, 8, 9 are supported.

You must specify the order you wish to run the program for **before** building
the program. (A rebuild is needed everytime you change the order.) This order
is specified on line 8 of the file `options.h`

When run without parameters, the program generates all *minimal* blocking sets
up to (projective) *equivalence*. 
The case of order 9 takes approximately 75 hours of computer time. 
You may however provide the program with parameters to indicate that you only want a part of the results.
This way the program can be run in parallel.

Typically you would run the following versions of the program in parallel:

    main 7 0 10
    main 7 1 10
    main 7 2 10
    ...
    main 7 9 10

The last parameter (10) indicates in how many parts the generation should be split,
the second parameter (0..9) indicates which of these parts you want to generate and the
first parameter (7) indicates at which depth the search tree should be split.
(Note that any blocking set with size smaller than or equal to this depth will be generated several times.)

#### Output format

The program generates some statistics on the *standard error* channel. The blocking sets themselves
are sent to *standard output*, one line per blocking set. Each line
lists the points as numbers terminated by dashes (e.g. `0-1-2-5-6-9-10-`). 

The points are
numbered according to their position in the orbit of a Singer cycle: we use the perfect 
difference set representation of the projective plane, i.e., a set of numbers D 
such that all differences a-b, with a,b ∈ D, a≠b are different modulo v = q²+q+1.

E.g q=2, v=7, D={0,1,3}.

* Points are numbered 0..v-1
* Lines are numbered 0..v-1
* Line l contains all points l+a (mod v), a ∈ D
* Point p goes through all lines p-a (mod v), a ∈ D

The specific difference set used for a plane of given order can be found in the 
corresponding file `pg2*.h` in `Main_library/planes`. 

#### Depends on Nauty

To be able to compile and run this program, download, install and 'make' a 
[Nauty distribution](https://pallini.di.uniroma1.it/) (currently we are using version `nauty26r5` ). 
Make a softlink to `nauty` in the base directory of this project.

#### Unit tests

To build and execute the unit tests you need a directory `Google_tests/lib` which is not included
in the repository. 
Clone Google Test from the [official repository](https://github.com/google/googletest). 
Link that folder (or make a copy) to `Google_tests/lib`.

 
